package arzumify.presence.matchmaker;

import arzumify.presence.maths.Vec2i;
import arzumify.presence.presences.Presence;
import arzumify.presence.presences.PresenceProvider;

import java.util.Collections;
import java.util.HashSet;
import java.util.Map;
import java.util.Set;
import java.util.concurrent.ConcurrentHashMap;

public class MatchMaker<P extends Presence<P>, T extends PresenceProvider<P>> {
    public MatchMaker() {
    }

    private final Map<Channel, Map<Vec2i, Set<T>>> channels = new ConcurrentHashMap<>();

    public void Add(T match, Channel channel) {
        channels.putIfAbsent(channel, new ConcurrentHashMap<>());
        for (Vec2i chunk : match.presence().GetChunks()) {
            channels.get(channel).putIfAbsent(chunk, new HashSet<>());
            channels.get(channel).get(chunk).add(match);
        }
    }

    public Set<T> Search(T match, Channel channel) {
        var matches = new HashSet<T>();
        var presenceChunks = match.presence().GetCollisionChunks();
        var chunks = channels.get(channel);
        if (chunks != null) {
            if (!Collections.disjoint(presenceChunks, chunks.keySet())) {
                presenceChunks.retainAll(chunks.keySet());
                for (Vec2i chunk : presenceChunks) {
                    if (chunks.get(chunk) != null) {
                        for (T matching : chunks.get(chunk)) {
                            if (match != matching) {
                                if (match.presence().Matches(matching.presence())) {
                                    matches.add(matching);
                                }
                            }
                        }
                    }
                }
            }
            return matches;
        } else {
            return null;
        }
    }

    public void Remove(T match, Channel channel) {
        for (Vec2i chunk : match.presence().GetChunks()) {
            if (channels.get(channel) != null) {
                if (channels.get(channel).get(chunk) != null) {
                    channels.get(channel).get(chunk).remove(match);
                }
            }
        }
    }
}